<?php
/**
 * Plugin Name: Member Profile Fields for WishList Member and Gravity Forms User Registration Add-On
 * Description: Allows setting WishList Member Fields when users are automatically created using Gravity Forms User Registration Add-On.
 * Version: 1.00
 * Author: AoD Technologies LLC
 * Author URI: https://aod-tech.com/
 * License: GPL-3.0+
 * License URI: https://www.gnu.org/licenses/gpl-3.0.txt
 * Text Domain: memberprofilefieldsforwlmandgfuserregistration
 * Domain Path: /languages
 *
 * Member Profile Fields for WishList Member and Gravity Forms User Registration Add-On Plugin
 * Copyright (C) 2020  AoD Technologies LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

namespace MemberProfileFieldsForWLMAndGFUserRegistration;

if ( !defined( 'ABSPATH' ) ) {
	die();
}

class Plugin {
	private $gf_user_registration = null;

	public function __construct() {
		add_action( 'init', array( $this, 'init' ) );
	}

	public function init() {
		if ( !class_exists( '\\GF_User_Registration' ) || !function_exists( '\\wlmapi_update_member' ) ) {
			// Gravity Forms User Registration Add-On and/or WishList Member not activated, so we do nothing
			return;
		}

		$this->gf_user_registration = \GF_User_Registration::get_instance();

		add_filter( 'gform_userregistration_feed_settings_fields', array( $this, 'filter_feed_settings_fields' ) );

		add_action( 'gform_user_registered', array( $this, 'update_wlm_data' ), 5, 3 );
		add_action( 'gform_user_updated',    array( $this, 'update_wlm_data' ), 5, 3 );
	}

	public function filter_feed_settings_fields( $fields ) {
		$num_fields = count( $fields );
		if ( $num_fields === 0 ) {
			return $fields;
		}

		return array_merge(
			array_slice( $fields, 0, $num_fields - 1 ),
			array(
				'wlm_meta' => array(
					'title'       => esc_html__( 'WishList Member Profile', 'memberprofilefieldsforwlmandgfuserregistration' ),
					'description' => '',
					'dependency'  => array(
						'field'   => 'feedType',
						'values'  => '_notempty_'
					),
					'fields'      => array(
						array(
							'name'      => 'wlmMeta',
							'label'     => '',
							'type'      => 'dynamic_field_map',
							'field_map' => array(
								array(
									'label' => esc_html__( 'Select WishList Member Field', 'wlmprofileforgravityformsuserregistration' ),
									'value' => ''
								),
								array(
									'label'   => esc_html__( 'Standard User Fields', 'memberprofilefieldsforwlmandgfuserregistration' ),
									'choices' => array(
										array(
											'label' => esc_html__( 'Company', 'memberprofilefieldsforwlmandgfuserregistration' ),
											'value' => 'company'
										),
										array(
											'label' => esc_html__( 'Street Address', 'memberprofilefieldsforwlmandgfuserregistration' ),
											'value' => 'address1'
										),
										array(
											'label' => esc_html__( 'Address Line 2', 'memberprofilefieldsforwlmandgfuserregistration' ),
											'value' => 'address2'
										),
										array(
											'label' => esc_html__( 'City', 'memberprofilefieldsforwlmandgfuserregistration' ),
											'value' => 'city'
										),
										array(
											'label' => esc_html__( 'State / Province', 'memberprofilefieldsforwlmandgfuserregistration' ),
											'value' => 'state'
										),
										array(
											'label' => esc_html__( 'ZIP / Postal Code', 'memberprofilefieldsforwlmandgfuserregistration' ),
											'value' => 'zip'
										),
										array(
											'label' => esc_html__( 'Country', 'memberprofilefieldsforwlmandgfuserregistration' ),
											'value' => 'country'
										)
									)
								),
								array(
									'label' => esc_html__( 'Add Custom User Field', 'memberprofilefieldsforwlmandgfuserregistration' ),
									'value' => 'gf_custom'
								)
							),
							'class'     => 'medium'
						)
					)
				)
			),
			array_slice( $fields, $num_fields - 1 )
		);
	}

	public function update_wlm_data( $user_id, $feed, $entry ) {
		$wlm_data = $this->prepare_wlm_data( $user_id, $feed, $entry );

		$this->insert_wlm_data( $user_id, $wlm_data );
	}

	private function insert_wlm_data( $user_id, $wlm_data ) {
		if ( empty( $wlm_data ) ) {
			return;
		}

		\wlmapi_update_member( $user_id, $wlm_data );
	}

	private function prepare_wlm_data( $user_id, $feed, $entry ) {
		$wlm_data = array();
		$meta     = $this->get_wlm_meta( $feed );

		if ( empty( $meta ) ) {
			return $wlm_data;
		}

		$user_info = get_userdata( $user_id );
		if ( $user_info === false ) {
			return $wlm_data;
		}

		$form = \GFFormsModel::get_form_meta( $entry['form_id'] );

		foreach ( $meta as $wlm_field_id => $gf_field_id ) {
			if ( empty( $wlm_field_id ) || empty( $gf_field_id ) ) {
				continue;
			}

			$meta_value = $this->gf_user_registration->get_meta_value( $wlm_field_id, $meta, $form, $entry );

			if ( !in_array( $wlm_field_id, array( 'company', 'address1', 'address2', 'city', 'state', 'zip', 'country' ) ) && strpos( $wlm_field_id, 'custom_' ) !== 0 ) {
				$wlm_field_id = 'custom_' . $wlm_field_id;
			}

			$wlm_data[$wlm_field_id] = $meta_value;
		}

		$wlm_data['user_login'] = $user_info->user_login;
		$wlm_data['user_email'] = $user_info->user_email;

		return $wlm_data;
	}

	private function get_wlm_meta( $feed ) {
		return $this->gf_user_registration->prepare_dynamic_meta( \rgars( $feed, 'meta/wlmMeta' ) );
	}
}

new Plugin();

